/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <QGraphicsItem>
#include <QGraphicsSceneMouseEvent>
#include "interfaces/monocle/iannotation.h"

namespace LeechCraft
{
namespace Monocle
{
	class AnnBaseItem
	{
	protected:
		const IAnnotation_ptr BaseAnn_;
	public:
		AnnBaseItem (const IAnnotation_ptr& ann)
		: BaseAnn_ { ann }
		{
		}

		QGraphicsItem* GetItem ()
		{
			return dynamic_cast<QGraphicsItem*> (this);
		}

		virtual void UpdateRect (const QRectF& rect) = 0;
	};

	AnnBaseItem* MakeItem (const IAnnotation_ptr&, QGraphicsItem*);

	template<typename T>
	class AnnBaseGraphicsItem : public AnnBaseItem
							  , public T
	{
		QPointF PressedPos_;
	public:
		AnnBaseGraphicsItem (const IAnnotation_ptr& ann, QGraphicsItem *parent)
		: AnnBaseItem { ann }
		, T { parent }
		{
		}
	protected:
		void mousePressEvent (QGraphicsSceneMouseEvent *event)
		{
			PressedPos_ = event->pos ();
		}

		void mouseReleaseEvent (QGraphicsSceneMouseEvent *event)
		{
			if ((event->pos () - PressedPos_).manhattanLength () < 4)
			{
				// TODO
			}
		}
	};

	class TextAnnItem : public AnnBaseGraphicsItem<QGraphicsRectItem>
	{
		const ITextAnnotation_ptr Ann_;
	public:
		TextAnnItem (const ITextAnnotation_ptr&, QGraphicsItem*);

		void UpdateRect (const QRectF& rect);
	};

	class HighAnnItem : public AnnBaseGraphicsItem<QGraphicsItemGroup>
	{
		const IHighlightAnnotation_ptr Ann_;

		struct PolyData
		{
			QPolygonF Poly_;
			QGraphicsPolygonItem *Item_;
		};
		const QList<PolyData> Polys_;

		QRectF Bounding_;
	public:
		HighAnnItem (const IHighlightAnnotation_ptr&, QGraphicsItem*);

		void UpdateRect (const QRectF& rect);
	private:
		static QList<PolyData> ToPolyData (const QList<QPolygonF>&);
	};
}
}
